@@ -6,7 +6,15 @@
 {-# LANGUAGE KindSignatures #-}
 {-# LANGUAGE DeriveFunctor #-}
 {-# LANGUAGE ScopedTypeVariables #-}
--- | Please see the README.md file for examples of using this API.
+{-|
+Please see the README.md file for examples of using this API.
+
+__Applications using this library MUST use @-threaded@__ to link with the
+threaded version of the RTS. 'spawnProcess' and the others spawn a Haskell
+thread that blocks until the process exits. Without @-threaded@ this will block
+/all/ the Haskell threads in your program until the spawned process exits.
+
+-}
 module System.Process.Typed
     ( -- * Types
       ProcessConfig
@@ -24,7 +32,9 @@ module System.Process.Typed
     , setStdout
     , setStderr
     , setWorkingDir
+    , setWorkingDirInherit
     , setEnv
+    , setEnvInherit
     , setCloseFds
     , setCreateGroup
     , setDelegateCtlc
@@ -35,7 +45,9 @@ module System.Process.Typed
 #endif
 #if MIN_VERSION_process(1, 4, 0) && !WINDOWS
     , setChildGroup
+    , setChildGroupInherit
     , setChildUser
+    , setChildUserInherit
 #endif
 
       -- * Stream specs
@@ -48,10 +60,6 @@ module System.Process.Typed
     , useHandleOpen
     , useHandleClose
 
-      -- ** Conduit
-    , createSink
-    , createSource
-
       -- * Launch a process
     , startProcess
     , stopProcess
@@ -61,6 +69,10 @@ module System.Process.Typed
     , readProcess_
     , runProcess
     , runProcess_
+    , readProcessStdout
+    , readProcessStdout_
+    , readProcessStderr
+    , readProcessStderr_
 
       -- * Interact with a process
 
@@ -86,11 +98,10 @@ module System.Process.Typed
 
 import qualified Data.ByteString as S
 import Data.ByteString.Lazy.Internal (defaultChunkSize)
-import Control.Exception (assert, evaluate, throwIO)
+import Control.Exception (assert, evaluate, throwIO, Exception, SomeException, finally, bracket, onException, catch)
 import Control.Monad (void)
 import Control.Monad.IO.Class
 import qualified System.Process as P
-import Control.Monad.Catch as C
 import Data.Typeable (Typeable)
 import System.IO (Handle, hClose)
 import Control.Concurrent.Async (async, cancel, waitCatch)
@@ -99,9 +110,6 @@ import System.Exit (ExitCode (ExitSuccess))
 import qualified Data.ByteString.Lazy as L
 import qualified Data.ByteString.Lazy.Char8 as L8
 import Data.String (IsString (fromString))
-import Data.Conduit (ConduitM)
-import qualified Data.Conduit as C
-import qualified Data.Conduit.Binary as CB
 
 #if MIN_VERSION_process(1, 4, 0) && !WINDOWS
 import System.Posix.Types (GroupID, UserID)
@@ -332,6 +340,14 @@ setWorkingDir :: FilePath
               -> ProcessConfig stdin stdout stderr
 setWorkingDir dir pc = pc { pcWorkingDir = Just dir }
 
+-- | Inherit the working directory from the parent process.
+--
+-- @since 0.2.2.0
+setWorkingDirInherit
+  :: ProcessConfig stdin stdout stderr
+  -> ProcessConfig stdin stdout stderr
+setWorkingDirInherit pc = pc { pcWorkingDir = Nothing }
+
 -- | Set the environment variables of the child process.
 --
 -- Default: current process's environment.
@@ -342,6 +358,14 @@ setEnv :: [(String, String)]
        -> ProcessConfig stdin stdout stderr
 setEnv env pc = pc { pcEnv = Just env }
 
+-- | Inherit the environment variables from the parent process.
+--
+-- @since 0.2.2.0
+setEnvInherit
+  :: ProcessConfig stdin stdout stderr
+  -> ProcessConfig stdin stdout stderr
+setEnvInherit pc = pc { pcEnv = Nothing }
+
 -- | Should we close all file descriptors besides stdin, stdout, and
 -- stderr? See 'P.close_fds' for more information.
 --
@@ -427,6 +451,14 @@ setChildGroup
     -> ProcessConfig stdin stdout stderr
 setChildGroup x pc = pc { pcChildGroup = Just x }
 
+-- | Inherit the group from the parent process.
+--
+-- @since 0.2.2.0
+setChildGroupInherit
+  :: ProcessConfig stdin stdout stderr
+  -> ProcessConfig stdin stdout stderr
+setChildGroupInherit pc = pc { pcChildGroup = Nothing }
+
 -- | Set the child process's user ID with the POSIX @setuid@ syscall,
 -- does nothing on non-POSIX. See 'P.child_user'.
 --
@@ -438,6 +470,14 @@ setChildUser
     -> ProcessConfig stdin stdout stderr
     -> ProcessConfig stdin stdout stderr
 setChildUser x pc = pc { pcChildUser = Just x }
+
+-- | Inherit the user from the parent process.
+--
+-- @since 0.2.2.0
+setChildUserInherit
+  :: ProcessConfig stdin stdout stderr
+  -> ProcessConfig stdin stdout stderr
+setChildUserInherit pc = pc { pcChildUser = Nothing }
 #endif
 
 -- | Create a new 'StreamSpec' from the given 'P.StdStream' and a
@@ -538,22 +578,6 @@ useHandleOpen h = mkStreamSpec (P.UseHandle h) $ \_ Nothing -> return ((), retur
 useHandleClose :: Handle -> StreamSpec anyStreamType ()
 useHandleClose h = mkStreamSpec (P.UseHandle h) $ \_ Nothing -> return ((), hClose h)
 
--- | Provide input to a process by writing to a conduit.
---
--- @since 0.1.0.0
-createSink :: MonadIO m => StreamSpec 'STInput (ConduitM S.ByteString o m ())
-createSink =
-    (\h -> C.addCleanup (\_ -> liftIO $ hClose h) (CB.sinkHandle h))
-    <$> createPipe
-
--- | Read output from a process by read from a conduit.
---
--- @since 0.1.0.0
-createSource :: MonadIO m => StreamSpec 'STOutput (ConduitM i S.ByteString m ())
-createSource =
-    (\h -> C.addCleanup (\_ -> liftIO $ hClose h) (CB.sourceHandle h))
-    <$> createPipe
-
 -- | Launch a process based on the given 'ProcessConfig'. You should
 -- ensure that you close 'stopProcess' on the result. It's usually
 -- better to use one of the functions in this module which ensures
@@ -642,24 +666,28 @@ stopProcess :: MonadIO m
             -> m ()
 stopProcess = liftIO . pCleanup
 
--- | Use the bracket pattern to call 'startProcess' and ensure
+-- | Uses the bracket pattern to call 'startProcess' and ensures that
 -- 'stopProcess' is called.
 --
+-- In version 0.2.0.0, this function was monomorphized to @IO@ to
+-- avoid a dependency on the exceptions package.
+--
 -- @since 0.1.0.0
-withProcess :: (MonadIO m, C.MonadMask m)
-            => ProcessConfig stdin stdout stderr
-            -> (Process stdin stdout stderr -> m a)
-            -> m a
-withProcess config = C.bracket (startProcess config) stopProcess
+withProcess :: ProcessConfig stdin stdout stderr
+            -> (Process stdin stdout stderr -> IO a)
+            -> IO a
+withProcess config = bracket (startProcess config) stopProcess
 
 -- | Same as 'withProcess', but also calls 'checkExitCode'
 --
+-- In version 0.2.0.0, this function was monomorphized to @IO@ to
+-- avoid a dependency on the exceptions package.
+--
 -- @since 0.1.0.0
-withProcess_ :: (MonadIO m, C.MonadMask m)
-             => ProcessConfig stdin stdout stderr
-             -> (Process stdin stdout stderr -> m a)
-             -> m a
-withProcess_ config = C.bracket
+withProcess_ :: ProcessConfig stdin stdout stderr
+             -> (Process stdin stdout stderr -> IO a)
+             -> IO a
+withProcess_ config = bracket
     (startProcess config)
     (\p -> stopProcess p `finally` checkExitCode p)
 
@@ -703,6 +731,70 @@ readProcess_ pc =
     pc' = setStdout byteStringOutput
         $ setStderr byteStringOutput pc
 
+-- | Same as 'readProcess', but only read the stdout of the process. Original settings for stderr remain.
+--
+-- @since 0.2.1.0
+readProcessStdout
+  :: MonadIO m
+  => ProcessConfig stdin stdoutIgnored stderr
+  -> m (ExitCode, L.ByteString)
+readProcessStdout pc =
+    liftIO $ withProcess pc' $ \p -> atomically $ (,)
+        <$> waitExitCodeSTM p
+        <*> getStdout p
+  where
+    pc' = setStdout byteStringOutput pc
+
+-- | Same as 'readProcessStdout', but instead of returning the
+-- 'ExitCode', checks it with 'checkExitCode'.
+--
+-- @since 0.2.1.0
+readProcessStdout_
+  :: MonadIO m
+  => ProcessConfig stdin stdoutIgnored stderr
+  -> m L.ByteString
+readProcessStdout_ pc =
+    liftIO $ withProcess pc' $ \p -> atomically $ do
+        stdout <- getStdout p
+        checkExitCodeSTM p `catchSTM` \ece -> throwSTM ece
+            { eceStdout = stdout
+            }
+        return stdout
+  where
+    pc' = setStdout byteStringOutput pc
+
+-- | Same as 'readProcess', but only read the stderr of the process. Original settings for stderr remain.
+--
+-- @since 0.2.1.0
+readProcessStderr
+  :: MonadIO m
+  => ProcessConfig stdin stderrIgnored stderr
+  -> m (ExitCode, L.ByteString)
+readProcessStderr pc =
+    liftIO $ withProcess pc' $ \p -> atomically $ (,)
+        <$> waitExitCodeSTM p
+        <*> getStderr p
+  where
+    pc' = setStderr byteStringOutput pc
+
+-- | Same as 'readProcessStderr', but instead of returning the
+-- 'ExitCode', checks it with 'checkExitCode'.
+--
+-- @since 0.2.1.0
+readProcessStderr_
+  :: MonadIO m
+  => ProcessConfig stdin stderrIgnored stderr
+  -> m L.ByteString
+readProcessStderr_ pc =
+    liftIO $ withProcess pc' $ \p -> atomically $ do
+        stderr <- getStderr p
+        checkExitCodeSTM p `catchSTM` \ece -> throwSTM ece
+            { eceStderr = stderr
+            }
+        return stderr
+  where
+    pc' = setStderr byteStringOutput pc
+
 -- | Run the given process, wait for it to exit, and returns its
 -- 'ExitCode'.
 --
@@ -712,7 +804,8 @@ runProcess :: MonadIO m
            -> m ExitCode
 runProcess pc = liftIO $ withProcess pc waitExitCode
 
--- | Same as 'runProcess', but ignores the 'ExitCode'.
+-- | Same as 'runProcess', but instead of returning the
+-- 'ExitCode', checks it with 'checkExitCode'.
 --
 -- @since 0.1.0.0
 runProcess_ :: MonadIO m
@@ -810,7 +903,9 @@ instance Show ExitCodeException where
         [ "Received "
         , show (eceExitCode ece)
         , " when running\n"
-        , show (eceProcessConfig ece)
+        -- Too much output for an exception if we show the modified
+        -- environment, so hide it
+        , show (eceProcessConfig ece) { pcEnv = Nothing }
         , if L.null (eceStdout ece)
             then ""
             else "Standard output:\n\n" ++ L8.unpack (eceStdout ece)
